@@ -1,69 +1,139 @@
 import pandas as pd
 
+from py_replay_bg.sensors import CGM
 
-class ModalityValidator:
+
+class BasalHandlerValidator:
     """
-    Class for validating the 'modality' input parameter of ReplayBG.
+    Class for validating the 'basal_handler' input parameter of ReplayBG.
     """
 
-    def __init__(self, modality):
-        self.modality = modality
+    def __init__(self, basal_handler):
+        self.basal_handler = basal_handler
 
     def validate(self):
-        if not (self.modality == 'identification' or self.modality == 'replay'):
-            raise Exception("'modality' input must be 'identification' or 'replay'.")
+        if not callable(self.basal_handler):
+            raise Exception("'basal_handler' input must be a function.'")
+
+
+class BasalHandlerParamsValidator:
+    """
+    Class for validating the 'basal_handler_params' input parameter of ReplayBG.
+    """
+
+    def __init__(self, basal_handler_params):
+        self.basal_handler_params = basal_handler_params
+
+    def validate(self):
+        if self.basal_handler_params is not None:
+            if not isinstance(self.basal_handler_params, dict):
+                raise Exception("'basal_handler_params' input must be a dict.'")
+
+
+class BasalSourceValidator:
+    """
+    Class for validating the 'basal_source' input parameter of ReplayBG.
+    """
+
+    def __init__(self, basal_source):
+        self.basal_source = basal_source
+
+    def validate(self):
+        if not (self.basal_source == 'data' or self.basal_source == 'u2ss' or self.basal_source == 'dss'):
+            raise Exception("'basal_source' input must be 'data', 'u2ss', or 'dss'.")
+
+
+class BolusCalculatorHandlerValidator:
+    """
+    Class for validating the 'bolus_calculator_handler' input parameter of ReplayBG.
+    """
+
+    def __init__(self, bolus_calculator_handler):
+        self.bolus_calculator_handler = bolus_calculator_handler
+
+    def validate(self):
+        if not callable(self.bolus_calculator_handler):
+            raise Exception("'bolus_calculator_handler' input must be a function.'")
+
+
+class BolusCalculatorHandlerParamsValidator:
+    """
+    Class for validating the 'bolus_calculator_handler_params' input parameter of ReplayBG.
+    """
+
+    def __init__(self, bolus_calculator_handler_params):
+        self.bolus_calculator_handler_params = bolus_calculator_handler_params
+
+    def validate(self):
+        if self.bolus_calculator_handler_params is not None:
+            if not isinstance(self.bolus_calculator_handler_params, dict):
+                raise Exception("'bolus_calculator_handler_params' input must be a dict.'")
+
+
+class BolusSourceValidator:
+    """
+    Class for validating the 'bolus_source' input parameter of ReplayBG.
+    """
+
+    def __init__(self, bolus_source):
+        self.bolus_source = bolus_source
+
+    def validate(self):
+        if not (self.bolus_source == 'data' or self.bolus_source == 'dss'):
+            raise Exception("'bolus_source' input must be 'data' or 'dss'.")
 
 
 class BWValidator:
     """
-    Class for validating the 'BW' input parameter of ReplayBG.
+    Class for validating the 'bw' input parameter of ReplayBG.
     """
 
-    def __init__(self, BW):
-        self.BW = BW
+    def __init__(self, bw):
+        self.bw = bw
 
     def validate(self):
-        if not (isinstance(self.BW, float) or isinstance(self.BW, int)):
-            raise Exception("'BW' input must be a number.'")
+        if not (isinstance(self.bw, float) or isinstance(self.bw, int)):
+            raise Exception("'bw' input must be a number.'")
 
 
-class ScenarioValidator:
+class CHOSourceValidator:
     """
-    Class for validating the 'scenario' input parameter of ReplayBG.
+    Class for validating the 'cho_source' input parameter of ReplayBG.
     """
 
-    def __init__(self, scenario):
-        self.scenario = scenario
+    def __init__(self, cho_source):
+        self.cho_source = cho_source
 
     def validate(self):
-        if not (self.scenario == 'single-meal' or self.scenario == 'multi-meal'):
-            raise Exception("'scenario' input must be 'single-meal' or 'multi-meal'.")
+        if not (self.cho_source == 'data' or self.cho_source == 'generated'):
+            raise Exception("'cho_source' input must be 'data' or 'generated'.")
 
 
-class ExerciseValidator:
+class CorrectionBolusesHandlerValidator:
     """
-    Class for validating the 'exercise' input parameter of ReplayBG.
+    Class for validating the 'correction_boluses_handler' input parameter of ReplayBG.
     """
 
-    def __init__(self, exercise):
-        self.exercise = exercise
+    def __init__(self, correction_boluses_handler):
+        self.correction_boluses_handler = correction_boluses_handler
 
     def validate(self):
-        if not isinstance(self.exercise, bool):
-            raise Exception("'exercise' input must be a boolean.'")
+        if not callable(self.correction_boluses_handler):
+            raise Exception("'correction_boluses_handler' input must be a function.'")
 
 
-class SaveNameValidator:
+class CorrectionBolusesHandlerParamsValidator:
     """
-    Class for validating the 'save_name' input parameter of ReplayBG.
+    Class for validating the 'hypotreatments_handler_params' input parameter of ReplayBG.
     """
 
-    def __init__(self, save_name):
-        self.save_name = save_name
+    def __init__(self, correction_boluses_handler_params):
+        self.correction_boluses_handler_params = correction_boluses_handler_params
 
     def validate(self):
-        if not isinstance(self.save_name, str):
-            raise Exception("'save_name' input must be a string.'")
+        if self.correction_boluses_handler_params is not None:
+            if not isinstance(self.correction_boluses_handler_params, dict):
+                raise Exception("'correction_boluses_handler_params' input must be a dict.'")
 
 
 class DataValidator:
@@ -71,11 +141,11 @@ class DataValidator:
     Class for validating the 'data' input parameter of ReplayBG.
     """
 
-    def __init__(self, modality, data, scenario, exercise,
+    def __init__(self, modality, data, blueprint, exercise,
                  bolus_source, basal_source, cho_source):
         self.modality = modality
         self.data = data
-        self.scenario = scenario
+        self.blueprint = blueprint
         self.exercise = exercise
 
         self.bolus_source = bolus_source
@@ -90,7 +160,7 @@ class DataValidator:
         if not 't' in self.data:
             raise Exception("'data' must contain the 't' column.'")
 
-        if self.modality == 'identification':
+        if self.modality == 'twin':
 
             if not 'glucose' in self.data:
                 raise Exception("'data' must contain the 'glucose' column.'")
@@ -117,7 +187,7 @@ class DataValidator:
                 if self.data.exercise.isnull().values.any():
                     raise Exception("'data.exercise' must not contain nan values.'")
 
-            if self.scenario == 'multi-meal':
+            if self.blueprint == 'multi-meal':
                 # TODO: implement multi-meal extra checks
                 # d = {'t': t, 'glucose': glucose, 'cho': cho, 'choLabel' : choLabel, 'bolus' : bolus, 'bolusLabel' : bolusLabel, 'basal' : basal, 'exercise' : exercise}
                 pass
@@ -140,278 +210,17 @@ class DataValidator:
                 raise Exception("'data.basal' must not contain nan values.'")
 
 
-class SaveSuffixValidator:
+class EnableCorrectionBolusesValidator:
     """
-    Class for validating the 'save_suffix' input parameter of ReplayBG.
+    Class for validating the 'enable_correction_boluses' input parameter of ReplayBG.
     """
 
-    def __init__(self, save_suffix):
-        self.save_suffix = save_suffix
+    def __init__(self, enable_correction_boluses):
+        self.enable_correction_boluses = enable_correction_boluses
 
     def validate(self):
-        if not isinstance(self.save_suffix, str):
-            raise Exception("'save_suffix' input must be a string.'")
-
-
-class YTSValidator:
-    """
-    Class for validating the 'yts' input parameter of ReplayBG.
-    """
-
-    def __init__(self, yts):
-        self.yts = yts
-
-    def validate(self):
-        if not isinstance(self.yts, int):
-            raise Exception("'yts' input must be an integer.'")
-
-
-class GlucoseModelValidator:
-    """
-    Class for validating the 'glucose_model' input parameter of ReplayBG.
-    """
-
-    def __init__(self, glucose_model):
-        self.glucose_model = glucose_model
-
-    def validate(self):
-        if not (self.glucose_model == 'BG' or self.glucose_model == 'IG'):
-            raise Exception("'glucose_model' input must be 'BG' or 'IG'.")
-
-
-class PathologyValidator:
-    """
-    Class for validating the 'pathology' input parameter of ReplayBG.
-    """
-
-    def __init__(self, pathology):
-        self.pathology = pathology
-
-    def validate(self):
-        if not (
-                self.pathology == 't1d' or self.pathology == 't2d' or self.pathology == 'pbh' or self.pathology == 'healthy'):
-            raise Exception("'pathology' input must be 't1d', 't2d', 'pbh', or 'healthy'.")
-
-
-class SeedValidator:
-    """
-    Class for validating the 'seed' input parameter of ReplayBG.
-    """
-
-    def __init__(self, seed):
-        self.seed = seed
-
-    def validate(self):
-        if not isinstance(self.seed, int):
-            raise Exception("'seed' input must be an integer.'")
-
-
-class BolusSourceValidator:
-    """
-    Class for validating the 'bolus_source' input parameter of ReplayBG.
-    """
-
-    def __init__(self, bolus_source):
-        self.bolus_source = bolus_source
-
-    def validate(self):
-        if not (self.bolus_source == 'data' or self.bolus_source == 'dss'):
-            raise Exception("'bolus_source' input must be 'data' or 'dss'.")
-
-
-class BasalSourceValidator:
-    """
-    Class for validating the 'basal_source' input parameter of ReplayBG.
-    """
-
-    def __init__(self, basal_source):
-        self.basal_source = basal_source
-
-    def validate(self):
-        if not (self.basal_source == 'data' or self.basal_source == 'u2ss' or self.basal_source == 'dss'):
-            raise Exception("'basal_source' input must be 'data', 'u2ss', or 'dss'.")
-
-
-class CHOSourceValidator:
-    """
-    Class for validating the 'cho_source' input parameter of ReplayBG.
-    """
-
-    def __init__(self, cho_source):
-        self.cho_source = cho_source
-
-    def validate(self):
-        if not (self.cho_source == 'data' or self.cho_source == 'generated'):
-            raise Exception("'cho_source' input must be 'data' or 'generated'.")
-
-
-class CGMModelValidator:
-    """
-    Class for validating the 'cgm_model' input parameter of ReplayBG.
-    """
-
-    def __init__(self, cgm_model):
-        self.cgm_model = cgm_model
-
-    def validate(self):
-        if not (self.cgm_model == 'CGM' or self.cgm_model == 'IG'):
-            raise Exception("'cgm_model' input must be 'CGM' or 'IG'.")
-
-
-class NStepsValidator:
-    """
-    Class for validating the 'n_steps' input parameter of ReplayBG.
-    """
-
-    def __init__(self, n_steps):
-        self.n_steps = n_steps
-
-    def validate(self):
-        if not isinstance(self.n_steps, int):
-            raise Exception("'n_steps' input must be an integer.'")
-
-
-class ToSampleValidator:
-    """
-    Class for validating the 'to_sample' input parameter of ReplayBG.
-    """
-
-    def __init__(self, to_sample):
-        self.to_sample = to_sample
-
-    def validate(self):
-        if not isinstance(self.to_sample, int):
-            raise Exception("'to_sample' input must be an integer.'")
-
-
-class SaveChainsValidator:
-    """
-    Class for validating the 'save_chains' input parameter of ReplayBG.
-    """
-
-    def __init__(self, save_chains):
-        self.save_chains = save_chains
-
-    def validate(self):
-        if not isinstance(self.save_chains, bool):
-            raise Exception("'save_chains' input must be a boolean.'")
-
-
-class CRValidator:
-    """
-    Class for validating the 'CR' input parameter of ReplayBG.
-    """
-
-    def __init__(self, CR):
-        self.CR = CR
-
-    def validate(self):
-        if not (isinstance(self.CR, float) or isinstance(self.CR, int)):
-            raise Exception("'CR' input must be a number.'")
-
-
-class CFValidator:
-    """
-    Class for validating the 'CF' input parameter of ReplayBG.
-    """
-
-    def __init__(self, CF):
-        self.CF = CF
-
-    def validate(self):
-        if not (isinstance(self.CF, float) or isinstance(self.CF, int)):
-            raise Exception("'CF' input must be a number.'")
-
-
-class GTValidator:
-    """
-    Class for validating the 'GT' input parameter of ReplayBG.
-    """
-
-    def __init__(self, GT):
-        self.GT = GT
-
-    def validate(self):
-        if not (isinstance(self.GT, float) or isinstance(self.GT, int)):
-            raise Exception("'GT' input must be a number.'")
-
-
-class MealGeneratorHandlerValidator:
-    """
-    Class for validating the 'meal_generator_handler' input parameter of ReplayBG.
-    """
-
-    def __init__(self, meal_generator_handler):
-        self.meal_generator_handler = meal_generator_handler
-
-    def validate(self):
-        if not callable(self.meal_generator_handler):
-            raise Exception("'meal_generator_handler' input must be a function.'")
-
-
-class MealGeneratorHandlerParamsValidator:
-    """
-    Class for validating the 'meal_generator_handler_params' input parameter of ReplayBG.
-    """
-
-    def __init__(self, meal_generator_handler_params):
-        self.meal_generator_handler_params = meal_generator_handler_params
-
-    def validate(self):
-        if not isinstance(self.meal_generator_handler_params, dict):
-            raise Exception("'meal_generator_handler_params' input must be a dict.'")
-
-
-class BolusCalculatorHandlerValidator:
-    """
-    Class for validating the 'bolus_calculator_handler' input parameter of ReplayBG.
-    """
-
-    def __init__(self, bolus_calculator_handler):
-        self.bolus_calculator_handler = bolus_calculator_handler
-
-    def validate(self):
-        if not callable(self.bolus_calculator_handler):
-            raise Exception("'bolus_calculator_handler' input must be a function.'")
-
-
-class BolusCalculatorHandlerParamsValidator:
-    """
-    Class for validating the 'bolus_calculator_handler_params' input parameter of ReplayBG.
-    """
-
-    def __init__(self, bolus_calculator_handler_params):
-        self.bolus_calculator_handler_params = bolus_calculator_handler_params
-
-    def validate(self):
-        if not isinstance(self.bolus_calculator_handler_params, dict):
-            raise Exception("'bolus_calculator_handler_params' input must be a dict.'")
-
-
-class BasalHandlerValidator:
-    """
-    Class for validating the 'basal_handler' input parameter of ReplayBG.
-    """
-
-    def __init__(self, basal_handler):
-        self.basal_handler = basal_handler
-
-    def validate(self):
-        if not callable(self.basal_handler):
-            raise Exception("'basal_handler' input must be a function.'")
-
-
-class BasalHandlerParamsValidator:
-    """
-    Class for validating the 'basal_handler_params' input parameter of ReplayBG.
-    """
-
-    def __init__(self, basal_handler_params):
-        self.basal_handler_params = basal_handler_params
-
-    def validate(self):
-        if not isinstance(self.basal_handler_params, dict):
-            raise Exception("'basal_handler_params' input must be a dict.'")
+        if not isinstance(self.enable_correction_boluses, bool):
+            raise Exception("'enable_correction_boluses' input must be a boolean.'")
 
 
 class EnableHypotreatmentsValidator:
@@ -427,6 +236,45 @@ class EnableHypotreatmentsValidator:
             raise Exception("'enable_hypotreatments' input must be a boolean.'")
 
 
+class ExerciseValidator:
+    """
+    Class for validating the 'exercise' input parameter of ReplayBG.
+    """
+
+    def __init__(self, exercise):
+        self.exercise = exercise
+
+    def validate(self):
+        if not isinstance(self.exercise, bool):
+            raise Exception("'exercise' input must be a boolean.'")
+
+
+class ExtendedValidator:
+    """
+    Class for validating the 'extended' input parameter of ReplayBG.
+    """
+
+    def __init__(self, extended):
+        self.extended = extended
+
+    def validate(self):
+        if not isinstance(self.extended, bool):
+            raise Exception("'extended' input must be a boolean.'")
+
+
+class FindStartGuessFirstValidator:
+    """
+    Class for validating the 'find_start_guess_first' input parameter of ReplayBG.
+    """
+
+    def __init__(self, find_start_guess_first):
+        self.find_start_guess_first = find_start_guess_first
+
+    def validate(self):
+        if not isinstance(self.find_start_guess_first, bool):
+            raise Exception("'find_start_guess_first' input must be a boolean.'")
+
+
 class HypotreatmentsHandlerValidator:
     """
     Class for validating the 'hypotreatments_handler' input parameter of ReplayBG.
@@ -449,47 +297,103 @@ class HypotreatmentsHandlerParamsValidator:
         self.hypotreatments_handler_params = hypotreatments_handler_params
 
     def validate(self):
-        if not isinstance(self.hypotreatments_handler_params, dict):
-            raise Exception("'hypotreatments_handler_params' input must be a dict.'")
+        if self.hypotreatments_handler_params is not None:
+            if not isinstance(self.hypotreatments_handler_params, dict):
+                raise Exception("'hypotreatments_handler_params' input must be a dict.'")
 
 
-class EnableCorrectionBolusesValidator:
+class TwinningMethodValidator:
     """
-    Class for validating the 'enable_correction_boluses' input parameter of ReplayBG.
+    Class for validating the 'twinning_method' input parameter of ReplayBG.
     """
 
-    def __init__(self, enable_correction_boluses):
-        self.enable_correction_boluses = enable_correction_boluses
+    def __init__(self, twinning_method):
+        self.twinning_method = twinning_method
 
     def validate(self):
-        if not isinstance(self.enable_correction_boluses, bool):
-            raise Exception("'enable_correction_boluses' input must be a boolean.'")
+        if not (self.twinning_method == 'mcmc' or self.twinning_method == 'map'):
+            raise Exception("'twinning_method' input must be 'mcmc' or 'map'.")
 
 
-class CorrectionBolusesHandlerValidator:
+class MealGeneratorHandlerValidator:
     """
-    Class for validating the 'correction_boluses_handler' input parameter of ReplayBG.
+    Class for validating the 'meal_generator_handler' input parameter of ReplayBG.
     """
 
-    def __init__(self, correction_boluses_handler):
-        self.correction_boluses_handler = correction_boluses_handler
+    def __init__(self, meal_generator_handler):
+        self.meal_generator_handler = meal_generator_handler
 
     def validate(self):
-        if not callable(self.correction_boluses_handler):
-            raise Exception("'correction_boluses_handler' input must be a function.'")
+        if not callable(self.meal_generator_handler):
+            raise Exception("'meal_generator_handler' input must be a function.'")
 
 
-class CorrectionBolusesHandlerParamsValidator:
+class MealGeneratorHandlerParamsValidator:
     """
-    Class for validating the 'hypotreatments_handler_params' input parameter of ReplayBG.
+    Class for validating the 'meal_generator_handler_params' input parameter of ReplayBG.
     """
 
-    def __init__(self, correction_boluses_handler_params):
-        self.correction_boluses_handler_params = correction_boluses_handler_params
+    def __init__(self, meal_generator_handler_params):
+        self.meal_generator_handler_params = meal_generator_handler_params
 
     def validate(self):
-        if not isinstance(self.correction_boluses_handler_params, dict):
-            raise Exception("'correction_boluses_handler_params' input must be a dict.'")
+        if self.meal_generator_handler_params is not None:
+            if not isinstance(self.meal_generator_handler_params, dict):
+                raise Exception("'meal_generator_handler_params' input must be a dict.'")
+
+
+class ModalityValidator:
+    """
+    Class for validating the 'modality' input parameter of ReplayBG.
+    """
+
+    def __init__(self, modality):
+        self.modality = modality
+
+    def validate(self):
+        if not (self.modality == 'twinning' or self.modality == 'replay'):
+            raise Exception("'modality' input must be 'twinning' or 'replay'.")
+
+
+class NProcessesValidator:
+    """
+    Class for validating the 'n_processes' input parameter of ReplayBG.
+    """
+
+    def __init__(self, n_processes):
+        self.n_processes = n_processes
+
+    def validate(self):
+        if not (isinstance(self.n_processes, int) or self.n_processes is None):
+            raise Exception("'n_processes' input must be an integer or None.'")
+
+
+class NReplayValidator:
+    """
+    Class for validating the 'n_replay' input parameter of ReplayBG.
+    """
+
+    def __init__(self, n_replay):
+        self.n_replay = n_replay
+
+    def validate(self):
+        if not isinstance(self.n_replay, int):
+            raise Exception("'n_replay' input must be an integer.'")
+        if not (self.n_replay == 1 or self.n_replay == 10 or self.n_replay == 100 or self.n_replay == 1000):
+            raise Exception("'n_replay' input must be 1, 10, 100, or 1000.'")
+
+
+class NStepsValidator:
+    """
+    Class for validating the 'n_steps' input parameter of ReplayBG.
+    """
+
+    def __init__(self, n_steps):
+        self.n_steps = n_steps
+
+    def validate(self):
+        if not isinstance(self.n_steps, int):
+            raise Exception("'n_steps' input must be an integer.'")
 
 
 class ParallelizeValidator:
@@ -505,6 +409,20 @@ class ParallelizeValidator:
             raise Exception("'parallelize' input must be a boolean.'")
 
 
+class PathologyValidator:
+    """
+    Class for validating the 'pathology' input parameter of ReplayBG.
+    """
+
+    def __init__(self, pathology):
+        self.pathology = pathology
+
+    def validate(self):
+        if not (
+                self.pathology == 't1d' or self.pathology == 't2d' or self.pathology == 'pbh' or self.pathology == 'healthy'):
+            raise Exception("'pathology' input must be 't1d', 't2d', 'pbh', or 'healthy'.")
+
+
 class PlotModeValidator:
     """
     Class for validating the 'plot_mode' input parameter of ReplayBG.
@@ -518,6 +436,141 @@ class PlotModeValidator:
             raise Exception("'plot_mode' input must be a boolean.'")
 
 
+class PreviousDataNameValidator:
+    """
+    Class for validating the 'previous_data_name' input parameter of ReplayBG.
+    """
+
+    def __init__(self, previous_data_name):
+        self.previous_data_name = previous_data_name
+
+    def validate(self):
+        if self.previous_data_name is not None:
+            if not isinstance(self.previous_data_name, str):
+                raise Exception("'previous_data_name' input must be a string.'")
+
+
+class SaveChainsValidator:
+    """
+    Class for validating the 'save_chains' input parameter of ReplayBG.
+    """
+
+    def __init__(self, save_chains):
+        self.save_chains = save_chains
+
+    def validate(self):
+        if not isinstance(self.save_chains, bool):
+            raise Exception("'save_chains' input must be a boolean.'")
+
+
+class SaveFolderValidator:
+    """
+    Class for validating the 'save_folder' input parameter of ReplayBG.
+    """
+
+    def __init__(self,
+                 save_folder: str
+                 ):
+        self.save_folder = save_folder
+
+    def validate(self):
+        if not isinstance(self.save_folder, str):
+            raise Exception("'save_folder' input must be a string.'")
+
+
+class SaveNameValidator:
+    """
+    Class for validating the 'save_name' input parameter of ReplayBG.
+    """
+
+    def __init__(self, save_name):
+        self.save_name = save_name
+
+    def validate(self):
+        if not isinstance(self.save_name, str):
+            raise Exception("'save_name' input must be a string.'")
+
+
+class SaveSuffixValidator:
+    """
+    Class for validating the 'save_suffix' input parameter of ReplayBG.
+    """
+
+    def __init__(self, save_suffix):
+        self.save_suffix = save_suffix
+
+    def validate(self):
+        if not isinstance(self.save_suffix, str):
+            raise Exception("'save_suffix' input must be a string.'")
+
+
+class SaveWorkspaceValidator:
+    """
+    Class for validating the 'save_workspace' input parameter of ReplayBG.
+    """
+
+    def __init__(self, save_workspace):
+        self.save_workspace = save_workspace
+
+    def validate(self):
+        if not isinstance(self.save_workspace, bool):
+            raise Exception("'save_workspace' input must be a boolean.'")
+
+
+class BlueprintValidator:
+    """
+    Class for validating the 'blueprint' input parameter of ReplayBG.
+    """
+
+    def __init__(self, blueprint):
+        self.blueprint = blueprint
+
+    def validate(self):
+        if not (self.blueprint == 'single-meal' or self.blueprint == 'multi-meal'):
+            raise Exception("'blueprint' input must be 'single-meal' or 'multi-meal'.")
+
+
+class SeedValidator:
+    """
+    Class for validating the 'seed' input parameter of ReplayBG.
+    """
+
+    def __init__(self, seed):
+        self.seed = seed
+
+    def validate(self):
+        if not isinstance(self.seed, int):
+            raise Exception("'seed' input must be an integer.'")
+
+
+class SensorsValidator:
+    """
+    Class for validating the 'sensors' input parameter of ReplayBG.
+    """
+
+    def __init__(self, sensors):
+        self.sensors = sensors
+
+    def validate(self):
+        if self.sensors is not None:
+            if not isinstance(self.sensors, list):
+                raise Exception("'sensors' input must be None or a list.'")
+
+
+class U2SSValidator:
+    """
+    Class for validating the 'u2ss' input parameter of ReplayBG.
+    """
+
+    def __init__(self, u2ss):
+        self.u2ss = u2ss
+
+    def validate(self):
+        if self.u2ss is not None:
+            if not (isinstance(self.u2ss, float) or isinstance(self.u2ss, int)):
+                raise Exception("'bw' input must be a number.'")
+
+
 class VerboseValidator:
     """
     Class for validating the 'verbose' input parameter of ReplayBG.
@@ -531,188 +584,28 @@ class VerboseValidator:
             raise Exception("'verbose' input must be a boolean.'")
 
 
-class InputValidator:
+class X0Validator:
     """
-    Class for validating the input of ReplayBG
-
-    ...
-    Attributes
-    ----------
-    (same as ReplayBG)
-
-    Methods
-    -------
-    validate():
-        Run the input validation process.
+    Class for validating the 'x0' input parameter of ReplayBG.
     """
 
-    def __init__(self, modality, data, BW, scenario, save_name, save_suffix,
-                 yts, glucose_model, pathology, exercise, seed,
-                 bolus_source, basal_source, cho_source,
-                 cgm_model,
-                 n_steps, to_sample, save_chains,
-                 CR, CF, GT,
-                 meal_generator_handler, meal_generator_handler_params,
-                 bolus_calculator_handler, bolus_calculator_handler_params,
-                 basal_handler, basal_handler_params,
-                 enable_hypotreatments, hypotreatments_handler, hypotreatments_handler_params,
-                 enable_correction_boluses, correction_boluses_handler, correction_boluses_handler_params,
-                 parallelize, plot_mode, verbose):
-        self.modality = modality
-        self.data = data
-        self.BW = BW
-        self.scenario = scenario
-        self.save_name = save_name
-        self.save_suffix = save_suffix
-
-        self.yts = yts
-        self.glucose_model = glucose_model
-        self.pathology = pathology
-        self.exercise = exercise
-        self.seed = seed
-
-        self.bolus_source = bolus_source
-        self.basal_source = basal_source
-        self.cho_source = cho_source
-
-        self.cgm_model = cgm_model
-
-        self.n_steps = n_steps
-        self.to_sample = to_sample
-        self.save_chains = save_chains
-
-        self.CR = CR
-        self.CF = CF
-        self.GT = GT
-
-        self.meal_generator_handler = meal_generator_handler
-        self.meal_generator_handler_params = meal_generator_handler_params
-        self.bolus_calculator_handler = bolus_calculator_handler
-        self.bolus_calculator_handler_params = bolus_calculator_handler_params
-        self.basal_handler = basal_handler
-        self.basal_handler_params = basal_handler_params
-
-        self.enable_hypotreatments = enable_hypotreatments
-        self.hypotreatments_handler = hypotreatments_handler
-        self.hypotreatments_handler_params = hypotreatments_handler_params
-        self.enable_correction_boluses = enable_correction_boluses
-        self.correction_boluses_handler = correction_boluses_handler
-        self.correction_boluses_handler_params = correction_boluses_handler_params
-
-        self.parallelize = parallelize
-        self.plot_mode = plot_mode
-        self.verbose = verbose
+    def __init__(self, x0):
+        self.x0 = x0
 
     def validate(self):
-        # Validate the 'modality' input
-        ModalityValidator(modality=self.modality).validate()
+        if self.x0 is not None:
+            if not isinstance(self.x0, list):
+                raise Exception("'x0' input must be None or a list.'")
 
-        # Validate the 'BW' input
-        BWValidator(BW=self.BW).validate()
 
-        # Validate the 'scenario' input
-        ScenarioValidator(scenario=self.scenario).validate()
+class YTSValidator:
+    """
+    Class for validating the 'yts' input parameter of ReplayBG.
+    """
 
-        # Validate the 'save_name' input
-        SaveNameValidator(save_name=self.save_name).validate()
+    def __init__(self, yts):
+        self.yts = yts
 
-        # Validate the 'exercise' input
-        ExerciseValidator(exercise=self.exercise).validate()
-
-        # Validate the 'save_suffix' input
-        SaveSuffixValidator(save_suffix=self.save_suffix).validate()
-
-        # Validate the 'bolus_source' input
-        BolusSourceValidator(bolus_source=self.bolus_source).validate()
-
-        # Validate the 'basal_source' input
-        BasalSourceValidator(basal_source=self.basal_source).validate()
-
-        # Validate the 'bolus_source' input
-        CHOSourceValidator(cho_source=self.cho_source).validate()
-
-        # Validate the 'data' input
-        DataValidator(modality=self.modality, data=self.data, scenario=self.scenario, exercise=self.exercise,
-                      bolus_source=self.bolus_source, basal_source=self.basal_source,
-                      cho_source=self.cho_source).validate()
-
-        # Validate the 'yts' input
-        YTSValidator(yts=self.yts).validate()
-
-        # Validate the 'glucose_model' input
-        GlucoseModelValidator(glucose_model=self.glucose_model).validate()
-
-        # Validate the 'pathology' input
-        PathologyValidator(pathology=self.pathology).validate()
-
-        # Validate the 'seed' input
-        SeedValidator(seed=self.seed).validate()
-
-        # Validate the 'cgm_model' input
-        CGMModelValidator(cgm_model=self.cgm_model).validate()
-
-        # Validate the 'n_steps' input
-        NStepsValidator(n_steps=self.n_steps).validate()
-
-        # Validate the 'to_sample' input
-        ToSampleValidator(to_sample=self.to_sample).validate()
-
-        # Validate the 'save_chains' input
-        SaveChainsValidator(save_chains=self.save_chains).validate()
-
-        # Validate the 'CR' input
-        CRValidator(CR=self.CR).validate()
-
-        # Validate the 'CF' input
-        CFValidator(CF=self.CF).validate()
-
-        # Validate the 'GT' input
-        GTValidator(GT=self.GT).validate()
-
-        # Validate the 'meal_generator_handler' input
-        MealGeneratorHandlerValidator(meal_generator_handler=self.meal_generator_handler).validate()
-
-        # Validate the 'meal_generator_handler_params' input
-        MealGeneratorHandlerParamsValidator(meal_generator_handler_params=self.meal_generator_handler_params).validate()
-
-        # Validate the 'bolus_calculator_handler' input
-        BolusCalculatorHandlerValidator(bolus_calculator_handler=self.bolus_calculator_handler).validate()
-
-        # Validate the 'bolus_calculator_handler_params' input
-        BolusCalculatorHandlerParamsValidator(
-            bolus_calculator_handler_params=self.bolus_calculator_handler_params).validate()
-
-        # Validate the 'basal_handler' input
-        BasalHandlerValidator(basal_handler=self.basal_handler).validate()
-
-        # Validate the 'basal_handler_params' input
-        BasalHandlerParamsValidator(basal_handler_params=self.basal_handler_params).validate()
-
-        # Validate the 'enable_hypotreatments' input
-        EnableHypotreatmentsValidator(enable_hypotreatments=self.enable_hypotreatments).validate()
-
-        # Validate the 'hypotreatments_handler' input
-        HypotreatmentsHandlerValidator(hypotreatments_handler=self.hypotreatments_handler).validate()
-
-        # Validate the 'hypotreatments_handler_params' input
-        HypotreatmentsHandlerParamsValidator(
-            hypotreatments_handler_params=self.hypotreatments_handler_params).validate()
-
-        # Validate the 'enable_correction_boluses' input
-        EnableCorrectionBolusesValidator(enable_correction_boluses=self.enable_correction_boluses).validate()
-
-        # Validate the 'correction_boluses_handler' input
-        CorrectionBolusesHandlerValidator(correction_boluses_handler=self.correction_boluses_handler).validate()
-
-        # Validate the 'hypotreatments_handler_params' input
-        CorrectionBolusesHandlerParamsValidator(
-            correction_boluses_handler_params=self.correction_boluses_handler_params).validate()
-
-        # Validate the 'parallelize' input
-        ParallelizeValidator(parallelize=self.parallelize).validate()
-
-        # Validate the 'plot_mode' input
-        PlotModeValidator(plot_mode=self.plot_mode).validate()
-
-        # Validate the 'verbose' input
-        VerboseValidator(verbose=self.verbose).validate()
+    def validate(self):
+        if not isinstance(self.yts, int):
+            raise Exception("'yts' input must be an integer.'")
